/*
+------------------------------------------------------------------------------+
| NAME:                                                                        |
| anl_tmCreateSynonyms.sas                                                     |
|                                                                              |
| TYPE:                                                                        |
| SAS data step and macro.                                                     |
|                                                                              |
| DESCRIPTION:                                                                 |
|  This program finds mis-spelled words in the input text fields and maps them |
|  to their correct expected parent. For example, trin may get mapped to train.|
|  The newly created synonym list (typos mapped to correct words) is then      |
|  appended to the master synonym list specified.                              |
|  This program uses %textsyn macro shipped with SAS TMiner to find synonyms.  |
|  It also uses %anl_tmpunc macro that can be found in the uncompiled macros   |
|  folder at SAS Warranty Analysis installation area.                          |
|                                                                              |
|  This program subsets the input data set into chunks of &l_chunk onservations|
|  so that text mining procs do not run out of memory.                         |
|                                                                              |
|  IMPORTANT: If the input data set is broken into smaller subsets, you should |
|  insure that the resulting synonym list does not contain more than one parent| 
|  for any given term because this condition might produce unpredictable       |
|  results when you run Text Analysis.                                         |
|                                                                              |
| SAS VERSION:                                                                 |
| The SAS System version 9.1.3                                                 |
|                                                                              |
| INPUT:                                                                       |
|-------------------------+---------+------------------------------------------|
| Name                    |Type     |Description                               |
|-------------------------+---------+------------------------------------------|
| commentLib              |Macro Var| Name of library containing comment table |
| commentDS               |Macro Var| Name of data set containing comment      |
|                         |         | variables                                |
| commentVars             |Macro Var| Space-delimted names of comment variables|
| language                |Macro Var| Primary language of the comment text     |
| stopDS                  |Macro Var| Data set containing terms that will be   |
|                         |         | dropped from consideration in            |
|                         |         | libName.tableName format                 |
| synonymDS               |Macro Var| Master synonym list that will be updated |
|                         |         | by this macro in libName.tableName format|
| l_chunk                 |Macro Var| Number of obs in each subset of input data|
|                                                                              |
| OUTPUT:                                                                      |
|-------------------------+---------+------------------------------------------|
| Name                    |Type     |Description                               |
|-------------------------+---------+------------------------------------------|
| &synonymDS              |Data Set | Updated synonym data set                 |
|                                                                              |
| CALLS:                                                                       |
| Called By:                                                                   |
|  Manual process                                                              |
| Calls:                                                                       |
|  anl_tmpunc                                                                  |
|  textsyn                                                                     |
|                                                                              |
|                                                                              |
| PRECONDITIONS:                                                               |
|                                                                              |
|                                                                              |
| POSTCONDITIONS:                                                              |
|                                                                              |
| EXAMPLES:                                                                    |
| %anl_tmCreateSynonyms;                                                       |
|                                                                              |
| DEVELOPMENT/MAINTENANCE HISTORY:                                             |
| DATE       BY        NOTE                                                    |
| 11Jun2007  gunedu    Initial prototype                                       |
| 29Feb2008  gunedu    Added code to find system encoding. Modified the last   |
|                      proc sort to eliminate duplicate rows for the same terms|
| 11Mar2008  gunedu    Added %nrbquote and %str functions to %if statements    |
|                      to fix S0498291                                         |
| 27Mar2007  gunedu    Added comments regarding anl_tmpunc macro.              |
| 13Nov2009  gunedu    Added Hunter's logic for breaking input data into small |
|                      chunks.                                                 |
| 22Feb2010  gunedu    Moved %anl_tmPunc macro call to inside the DO loop      |
| 07Jun2011  sinsaa	   Replaced proc docparse by tgparse as the proc name is   |
| 					   changed in SAS9.3 version.							   |
+------------------------------------------------------------------------------+
+------------------------------------------------------------------------------+
| Copyright  2007 by SAS Institute Inc., Cary, NC 27513 USA                    |
| ---All Rights Reserved.                                                      |
+------------------------------------------------------------------------------+
*/
options mprint;

/*************Important:  Include anl_tmpunc.sas that can be found in ucmacros folder. **************/
%include '\ucmacros\anl_tmpunc.sas';

/* Define a libref for parmsl. */
*libname parmsl ' ';

/* Define the primary language of the text of the comment fields. */
%let language = ENGLISH; 

/* Define a libref for dictionary tables */
/*libname dict '/sas/config/Lev1/AppData/SASWarrantyAnalysis4.3/lookup/dictionaries' access=readonly;*/
libname dict 'G:\SAS\Config\Lev1\SASApp\SASEnvironment\SASWarrantyAnalysis4.3\SASMacro' access=readonly;

/* Define a macro variable for the libname where the comment table is located. */
%let l_dictionaryLib = dict;

/* Define a macro variable for the table containing comment fields. */
%let l_dictionaryDS = eng_dictionary; 

/*Define output location for synonymlist so that sub-zeros tables are not overwritten */
libname tm 'G:\SAS\Config\Lev1\SASApp\SASEnvironment\SASWarrantyAnalysis4.3\SASMacro';

/* Define a libref for warranty data mart. */
libname swadm ' ';

/*To calulate the number of observations*/
data _NULL_;
	if 0 then set swadm.comment nobs=n;
	call symputx('nrows',n);
	stop;
run;
%put numdoc=&nrows;

/* Define a macro variable for the libname where the comment table is located. */
%let commentLib = ;

/* Define a macro variable for the table containing comment fields. */
%let commentDS = ;

/* Define a macro variable for the columns containing text of the comments - use space delimter. */
%let commentVars = ;

/* Define the primary language of the text of the comment fields. */
%let language = english;

/* Define a macro variable for stop data set, if one exists. */
%let stopDS = parmsl.textStopList;

/* Define a macro variable for the master synonym data set that will be updated by this macro. */
/* If the data set does not exist, it will be created. */
%let synonymDS = parmsl.textSynonymList;

/* Number of obs in each chunk of data to be analized by proc docparse and the textsyn macro */
%let l_chunk=10000; 



%macro anl_tmCreateSynonyms;

   /* Find the system encoding and create a macro variable to store it. */
   /* This macro variable will be an input for DOCPARSE procedure. */

   %let encoding=%sysfunc(getoption(ENCODING));
   %let encoding=%qtrim(&encoding);

   %if %nrbquote(&encoding)=%str(WLATIN1) or %nrbquote(&encoding)=%str(LATIN1) %then
   %do;
      %let inxencoding = cp_1252; %let xmlencoding= windows-1252;
   %end;
   %else %if %nrbquote(&encoding)=%str(WLATIN2) or %nrbquote(&encoding)=%str(LATIN2) %then
   %do;
      %let inxencoding = cp_1250; %let xmlencoding=windows-1250;
   %end;
   %else %if %nrbquote(&encoding)=%str(BIG5) or %nrbquote(&encoding)=%str(MS-950) %then
   %do;
      %let inxencoding = big5; %let xmlencoding=big5;
   %end;
   %else %if %nrbquote(&encoding)=%str(EUC-KR) or %nrbquote(&encoding)=%str(MS-949) %then %do;
      %let inxencoding = euc_kr; %let xmlencoding=euc-kr;
   %end;
   %else %if %nrbquote(&encoding)=%str(SHIFT-JIS)  %then
   %do;
      %let inxencoding = shift_jis; %let xmlencoding=shift-jis;
   %end;
   %else %if %nrbquote(&encoding)=%str(UTF8)  %then
   %do;
      %let inxencoding = utf_8; %let xmlencoding=utf-8;
   %end;
   %else %if %nrbquote(&encoding)=%str(UTF-8)  %then %do;
      %let inxencoding = utf_8; %let xmlencoding=utf-8;
   %end;
   %else %if %nrbquote(&encoding)=%str(EUC-CN)  %then
   %do;
      %let inxencoding = euc_cn; %let xmlencoding=euc-cn;
   %end;
   %else
   %do;
      %let inxencoding = ; %let xmlencoding=;
   %end;

   /* set the length of the synonylist variables. if you reduce the size after go live you will loose info */
   proc sql;
       alter table &synonymDS 
          modify term char(40), 
               parent char(40),
             category char(20);
   quit;
   
   /* after doc parse runs ignore any term that is already in the stop list or in the synonym list */
   /* no need to send known synonyms through proc tmspell */
   data work.ignoreTerms;
      set &synonymDS(keep=term)
   	  %if &stopDS ne %then &stopDS;;
   run;

   proc sort data=work.ignoreTerms nodupkey;
       by term;
   run;

   %macro docparse(inds=);
      /* Run DOCPARSE procedure to parse the comment field and create KEY and OUT data sets. */

      proc tgparse
         data=&inds
         stop=work.ignoreTerms
         stemming=yes
         tagging=no
         entities=no
         ng=sub
         language="%lowcase(&language)"
         encoding="&inxencoding"
         key=key&i
         out=out&i
         ;
      var &&commentVar&i;
      select "Punct" "Num" /group="attributes"  drop;
	  select "Punct" "Mixed" /group="attributes"  drop;
      select "Punct" "Num" /drop;
	  select "Address" "Currency" "Date" "Time" "Internet" "Person" "Phone" "Prop_misc" "Measure" "Time_Period" "Percent" "Organization" "Location" "Title"/ group="entities" drop;
      run;
   %mend docparse;


   /* Find the number of comment variables to consider. */
   %let commentVars = %cmpres(&commentVars);
   %let numCommentVars = %eval(%sysfunc(count(&commentVars, %str( )))+1);


   /* Loop through each comment variable. */
   %do i=1 %to &numCommentVars;
      %let commentVar&i = %scan(&commentVars, &i);


      filename txtmine&i catalog "work.textsynonyms.&&commentVar&i...source";

      data _null_;
        file txtmine&i;
         if _n_=0 then set &commentLib..&commentDS nobs=obsnum;

         do firstobs=1 to obsnum by &l_chunk;
            lastobs=firstobs+ifn(firstobs+&l_chunk-1>obsnum,obsnum-firstobs,&l_chunk-1);
            put ;
            put '   %put APPL-NOTE: Beginning analysis of observations ' firstobs 'through ' lastobs 'of ' obsnum ';';
            put ;
            put '   %anl_tmPunc(var= &&commentVar&i,inds= &commentLib..&commentDS(keep=&&commentVar&i firstobs=' firstobs 'obs=' lastobs '),outds= WORK.&commentDS.&i);';
            put ;
            put '   %docparse(inds=%str(WORK.&commentDS.&i));';
            put ;

            put '   %textsyn(  maxsped=10';
 			put '            , numdocs=&nrows' ;
            put '            , mnpardoc= ';
            put '            , multipen=2';
			put '            , dictpen=2';
            put '            , mxchddoc= ';
			put '			 ,dict=&l_dictionaryLib..&l_dictionaryDS';
            put '            , termds=work.key&i';
            put '            , synds=work.SynonymDS) ;';
			

			put;
		
			

            put '   proc sort data=work.key&i;';
	        put '       by term role;';
		    put "       where keep='Y';";
            put '   run;';
			put;
			
/*            put '   proc tmspell data= work.key&i (where=(_ispar ne "+")) out= work.SynonymDS';*/
/*            put '       minparents=10 maxchildren=6 maxspedis=10 multipen=4';*/
/*            put '	    dict=&l_dictionaryLib..&l_dictionaryDS dictpen=3;';*/
/*            put '   run;';*/
/*            put;*/
			



            put '   proc append base=work.synonymDS&i data=work.synonymDS force;';
            put '   run;';
            put;
            put '   proc datasets lib=WORK nolist;';
            put '      delete synonymDS key&i out&i ;';
            put '      run;';
            put '   quit;';
            put;
            put '   %put APPL-NOTE: Ending analysis of observations ' firstobs 'through ' lastobs 'of ' obsnum ';';
            put;
          end;

         stop;
      run;

     %include txtmine&i;

      /* If a new synonym data set is created for the comment variable, append it to the */
      /* master synonym data set. */

      %if %sysfunc(exist(WORK.SynonymDS&i)) %then
      %do;

	   /* only add new terms in case parents have been modified in the existing list */
	     proc sql;
		    create table work.newTerms as
			  select term, parent, category
			  from work.SynonymDS&i
			  where term in ( 
                 select term
			        from work.SynonymDS&i
			        except
			        select term
			        from &synonymDS)
              ;
		 quit;

         proc append base=&synonymDS data=work.newTerms force;
         run;

         proc datasets lib=WORK nolist;
             delete SynonymDS&i newTerms;
         run;
         quit;

      %end;

   %end;  /* i=1 %to &numCommentVars */

   /* Eliminate duplicate records from the master synonym data set. */

   proc sort data=&synonymDS nodupkey;
      by term;
   run;

%mend anl_tmCreateSynonyms;

%anl_tmCreateSynonyms;

proc sql;

  /* create list of circular logic terms */
  create table work.removeterm as
     select distinct a.TERM, a.parent, a.category
     from parmsl.textsynonymlist a, parmsl.textsynonymlist b
     where a.term=b.parent;
	 
  /* remove circular logic terms from data set */
  create table parmsl.textsynonymlist as
     select  *
     from parmsl.textsynonymlist
     except 
        select *
	    from work.removeterm ;
  quit;

proc sort data=parmsl.textsynonymlist;
	by term parent;
run;
